# Hunter's Knife Arena Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/knife_arena/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load knife_arena
#
################################################################################
#import EventScripts
import es
#import Libraries
import gamethread
import playerlib
import time
import random
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Knife Arena"
info.version = "2.0.0"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Turns your server into a 1vs1 Knife Arena"
info.basename = "knife_arena"

hunter_knife_arena_ver   = info.version
hunter_knife_arena_text  = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Server Variables
knife_arena_limit        = es.ServerVar('knife_arena_limit', '0', 'Is there a limit on how often a player can respawn? ( 0 = off | >0 = limit )')
knife_arena_timelimit    = es.ServerVar('knife_arena_timelimit', '30', 'Whats the time limit of 1vs1 fights?')
knife_arena_health       = es.ServerVar('knife_arena_health', '100', 'The start health of players in a fight')

# Global Variables
knife_arena_areas        = {}
knife_arena_players      = {}
knife_arena_waitqueue    = {2:[],3:[]}
knife_arena_scores       = {2:0,3:0}
knife_arena_reset        = True

def load():
    public = es.ServerVar('hu_ka', info.version, info.name)
    public.makepublic()

    knife_arena_players = {}
    for userid in es.getUseridList():
        knife_arena_players[userid] = {'lives':int(knife_arena_limit),'fight':False,'spawns':0,'time':time.time()}

    gamethread.delayed(10, player_check)

    es.regsaycmd('!watch', 'knife_arena/watch', 'Knife Arena Watch')
    es.regsaycmd('!fight', 'knife_arena/fight', 'Knife Arena Fight')
    es.log(hunter_knife_arena_text)
    es.msg('#multi', '#green[KnifeArena] #defaultLoaded')
    
def unload():
    es.unregsaycmd('!watch')
    es.unregsaycmd('!fight')
    es.msg('#multi', '#green[KnifeArena] #defaultUnloaded')
    
def es_map_start(event_var):
    global knife_arena_areas, knife_arena_scores
    knife_arena_scores = {2:0,3:0}
    knife_arena_reset = True

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_knife_arena_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_knife_arena_text)

def player_activate(event_var):
    knife_arena_players[int(event_var['userid'])] = {'lives':int(knife_arena_limit),'fight':False,'spawns':0,'time':time.time(),'opponent':0}
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_knife_arena_text))
    gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[KnifeArena] #defaultThis server is running a knife-only area'))
    
def round_start(event_var):
    for areaid in knife_arena_areas:
        knife_arena_areas[areaid]['inuse'] = False
        knife_arena_areas[areaid]['time'] = int(knife_arena_timelimit)
    for userid in knife_arena_players:
        knife_arena_players[userid]['lives'] = int(knife_arena_limit)

def player_check():
    gamethread.delayed(10, player_check)
    for userid in knife_arena_players:
        if int(knife_arena_players[userid]['time']) < int(time.time()-10):
            player_reset(userid, knife_arena_players[userid]['spawns'])

def player_wait(userid, team):
    if not userid in knife_arena_waitqueue[team]:
        knife_arena_waitqueue[team].append(userid)
        
def player_unlock(areaid):
    if areaid in knife_arena_areas:
        knife_arena_areas[areaid]['inuse'] = False

def player_reset(userid, spawns):
    if es.exists('userid', userid):
        if userid in knife_arena_players:
            if knife_arena_players[userid]['spawns'] == spawns:
                knife_arena_players[userid]['spawns'] = knife_arena_players[userid]['spawns'] + 1
                player = playerlib.getPlayer(userid)
                if (int(player.attributes['teamid']) > 1) and (int(player.attributes['isdead']) == 0):
                    player = playerlib.getPlayer(userid)
                    player.set('cash', '0')
                    player.set('health', int(knife_arena_health))
                    areaid = knife_arena_players[userid]['fight']
                    knife_arena_players[userid]['time'] = time.time()
                    es.ServerCommand('es_xsetpos '+str(userid)+' '+str(knife_arena_players[userid]['start'][0])+' '+str(knife_arena_players[userid]['start'][1])+' '+str(knife_arena_players[userid]['start'][2]))
                    es.ServerCommand('est_stripplayer '+str(userid)+' 1')
                    es.centertell(userid, 'Searching new opponent...')
                    gamethread.delayed(1, player_wait, (int(userid), int(player.attributes['teamid'])))
                    gamethread.delayed(1, player_queue, (int(userid), int(knife_arena_players[userid]['spawns'])))
                    if areaid > 0:
                        gamethread.delayed(5, player_unlock, (areaid))

def player_fight(areaid, userid, userteam, opponentid, opponentteam):
    if es.exists('userid', userid) and es.exists('userid', opponentid) and (knife_arena_areas[areaid]['inuse']):
        player = playerlib.getPlayer(userid)
        opponentplayer = playerlib.getPlayer(opponentid)
        if (knife_arena_players[userid]['fight'] == areaid) and (knife_arena_players[opponentid]['fight'] == areaid) and (knife_arena_players[userid]['opponent'] == opponentid) and (knife_arena_players[opponentid]['opponent'] == userid):
            knife_arena_areas[areaid]['time'] = knife_arena_areas[areaid]['time'] - 1
            player.set('cash', '0')
            opponentplayer.set('cash', '0')
            if (int(player.attributes['teamid']) == userteam) and (int(opponentplayer.attributes['teamid']) == opponentteam):
                if int(knife_arena_areas[areaid]['time']) > 0:
                    knife_arena_players[userid]['time'] = time.time()
                    knife_arena_players[opponentid]['time'] = time.time()
                    es.centertell(userid, str(knife_arena_areas[areaid]['time'])+'/'+str(knife_arena_timelimit)+' remaining')
                    es.centertell(opponentid, str(knife_arena_areas[areaid]['time'])+'/'+str(knife_arena_timelimit)+' remaining')
                    gamethread.delayed(1, player_fight, (areaid, userid, userteam, opponentid, opponentteam))
                elif int(knife_arena_timelimit) == 0:
                    knife_arena_players[userid]['time'] = time.time()
                    knife_arena_players[opponentid]['time'] = time.time()
                    gamethread.delayed(1, player_fight, (areaid, userid, userteam, opponentid, opponentteam))
                else:
                    knife_arena_players[userid]['fight'] = False
                    knife_arena_players[userid]['time'] = time.time()
                    knife_arena_players[userid]['opponent'] = 0
                    knife_arena_players[opponentid]['fight'] = False
                    knife_arena_players[opponentid]['time'] = time.time()
                    knife_arena_players[opponentid]['opponent'] = 0
                    es.centertell(userid, 'Timeout! Searching another opponent...')
                    es.centertell(opponentid, 'Timeout! Searching another opponent...')
                    gamethread.delayed(1, player_reset, (userid, int(knife_arena_players[userid]['spawns'])))
                    gamethread.delayed(1, player_reset, (opponentid, int(knife_arena_players[opponentid]['spawns'])))
            else:
                knife_arena_players[userid]['fight'] = False
                knife_arena_players[userid]['time'] = time.time()
                knife_arena_players[userid]['opponent'] = 0
                knife_arena_players[opponentid]['fight'] = False
                knife_arena_players[opponentid]['time'] = time.time()
                knife_arena_players[opponentid]['opponent'] = 0
                es.centertell(userid, 'Searching new opponent...')
                es.centertell(opponentid, 'Searching new opponent...')
                gamethread.delayed(1, player_reset, (userid, int(knife_arena_players[userid]['spawns'])))
                gamethread.delayed(1, player_reset, (opponentid, int(knife_arena_players[opponentid]['spawns'])))
        else:
            if knife_arena_players[userid]['fight'] == areaid:
                if knife_arena_players[opponentid]['fight'] != 'dead':
                    knife_arena_players[userid]['fight'] = False
                    knife_arena_players[userid]['time'] = time.time()
                    knife_arena_players[userid]['opponent'] = 0
                    es.centertell(userid, 'Searching new opponent...')
                    gamethread.delayed(1, player_reset, (userid, int(knife_arena_players[userid]['spawns'])))
            elif knife_arena_players[userid]['fight'] == areaid:
                if knife_arena_players[opponentid]['fight'] != 'dead':
                    knife_arena_players[opponentid]['fight'] = False
                    knife_arena_players[opponentid]['time'] = time.time()
                    knife_arena_players[opponentid]['opponent'] = 0
                    es.centertell(opponentid, 'Searching new opponent...')
                    gamethread.delayed(1, player_reset, (opponentid, int(knife_arena_players[opponentid]['spawns'])))
    elif es.exists('userid', userid):
        player = playerlib.getPlayer(userid)
        player.set('cash', '0')
        player.set('health', int(knife_arena_health))
        knife_arena_players[userid]['fight'] = False
        knife_arena_players[userid]['time'] = time.time()
        knife_arena_players[userid]['opponent'] = 0
        es.centertell(userid, 'Searching new opponent...')
        gamethread.delayed(1, player_reset, (userid, int(knife_arena_players[userid]['spawns'])))
    elif es.exists('userid', opponentid):
        player = playerlib.getPlayer(opponentid)
        player.set('cash', '0')
        player.set('health', int(knife_arena_health))
        knife_arena_players[opponentid]['fight'] = False
        knife_arena_players[opponentid]['time'] = time.time()
        knife_arena_players[opponentid]['opponent'] = 0
        es.centertell(opponentid, 'Searching new opponent...')
        gamethread.delayed(1, player_reset, (opponentid, int(knife_arena_players[opponentid]['spawns'])))
        
def player_zone(userid, userteam, opponentid, opponentteam):
    if es.exists('userid', userid) and es.exists('userid', opponentid):
        free_areas = []
        for areaid in knife_arena_areas:
            if not knife_arena_areas[areaid]['inuse']:
                free_areas.append(areaid)
        if free_areas:
            random_areaid = random.choice(free_areas)
            if (random_areaid > 0) and (random_areaid in knife_arena_areas):
                knife_arena_areas[random_areaid]['inuse'] = True
                knife_arena_players[userid]['fight'] = random_areaid
                knife_arena_players[opponentid]['fight'] = random_areaid
                knife_arena_players[userid]['opponent'] = opponentid
                knife_arena_players[opponentid]['opponent'] = userid
                knife_arena_players[userid]['time'] = time.time()
                knife_arena_players[opponentid]['time'] = time.time()
                es.ServerCommand('es_xsetpos '+str(userid)+' '+str(knife_arena_areas[random_areaid][userteam]['origin'][0])+' '+str(knife_arena_areas[random_areaid][userteam]['origin'][1])+' '+str(knife_arena_areas[random_areaid][userteam]['origin'][2]))
                es.ServerCommand('es_xsetpos '+str(opponentid)+' '+str(knife_arena_areas[random_areaid][opponentteam]['origin'][0])+' '+str(knife_arena_areas[random_areaid][opponentteam]['origin'][1])+' '+str(knife_arena_areas[random_areaid][opponentteam]['origin'][2]))
                es.ServerCommand('es_xsetang '+str(userid)+' '+str(knife_arena_areas[random_areaid][userteam]['rotation'][0])+' '+str(knife_arena_areas[random_areaid][userteam]['rotation'][1])+' '+str(knife_arena_areas[random_areaid][userteam]['rotation'][2]))
                es.ServerCommand('es_xsetang '+str(opponentid)+' '+str(knife_arena_areas[random_areaid][opponentteam]['rotation'][0])+' '+str(knife_arena_areas[random_areaid][opponentteam]['rotation'][1])+' '+str(knife_arena_areas[random_areaid][opponentteam]['rotation'][2]))
                es.ServerCommand('est_freeze '+str(userid)+' 1')
                es.ServerCommand('est_freeze '+str(opponentid)+' 1')
                es.centertell(userid, 'Prepare to Fight...')
                es.centertell(opponentid, 'Prepare to Fight...')
                gamethread.delayed(1, es.centertell, (userid, 'Good Luck!'))
                gamethread.delayed(1, es.centertell, (opponentid, 'Good Luck!'))
                gamethread.delayed(1, es.ServerCommand, ('es_xgive '+str(userid)+' weapon_knife'))
                gamethread.delayed(1, es.ServerCommand, ('es_xgive '+str(opponentid)+' weapon_knife'))
                gamethread.delayed(2, es.playsound, (userid, 'ambient/misc/brass_bell_c.wav', 0.2))
                gamethread.delayed(2, es.playsound, (opponentid, 'ambient/misc/brass_bell_c.wav', 0.2))
                gamethread.delayed(2, es.ServerCommand, ('est_freeze '+str(userid)+' 0'))
                gamethread.delayed(2, es.ServerCommand, ('est_freeze '+str(opponentid)+' 0'))
                gamethread.delayed(2, player_fight, (random_areaid, userid, userteam, opponentid, opponentteam))
            else:
                gamethread.delayed(3, player_zone, (userid, userteam, opponentid, opponentteam))
        else:
            gamethread.delayed(3, player_zone, (userid, userteam, opponentid, opponentteam))
    elif es.exists('userid', userid):
        player = playerlib.getPlayer(userid)
        player.set('cash', '0')
        player.set('health', int(knife_arena_health))
        es.centertell(userid, 'Searching new opponent...')
        gamethread.delayed(1, player_wait, (userid, int(player.attributes['teamid'])))
        gamethread.delayed(1, player_queue, (userid, int(knife_arena_players[userid]['spawns'])))
    elif es.exists('userid', opponentid):
        player = playerlib.getPlayer(opponentid)
        player.set('cash', '0')
        player.set('health', int(knife_arena_health))
        es.centertell(opponentid, 'Searching new opponent...')
        gamethread.delayed(1, player_wait, (opponentid, int(player.attributes['teamid'])))
        gamethread.delayed(1, player_queue, (opponentid, int(knife_arena_players[opponentid]['spawns'])))

def player_queue(userid, spawns):
    if es.exists('userid', userid):
        if userid in knife_arena_players:
            if knife_arena_players[userid]['spawns'] == spawns:
                knife_arena_players[userid]['spawns'] = knife_arena_players[userid]['spawns'] + 1
                if knife_arena_players[userid]['fight'] == False:
                    player = playerlib.getPlayer(userid)
                    if (int(player.attributes['teamid']) > 1) and (int(player.attributes['isdead']) == 0):
                        knife_arena_players[userid]['time'] = time.time()
                        if userid in knife_arena_waitqueue[int(player.attributes['teamid'])]:
                            opponentid = 0
                            if int(player.attributes['teamid']) == 2:
                                if knife_arena_waitqueue[3]:
                                    opponentid = random.choice(knife_arena_waitqueue[3])
                            elif int(player.attributes['teamid']) == 3:
                                if knife_arena_waitqueue[2]:
                                    opponentid = random.choice(knife_arena_waitqueue[2])
                            if (opponentid > 0) and es.exists('userid', opponentid):
                                opponentplayer = playerlib.getPlayer(opponentid)
                                knife_arena_waitqueue[int(player.attributes['teamid'])].remove(userid)
                                knife_arena_waitqueue[int(opponentplayer.attributes['teamid'])].remove(opponentid)
                                knife_arena_players[userid]['fight'] = True
                                knife_arena_players[opponentid]['fight'] = True
                                es.centertell(userid, 'Found opponent...')
                                es.centertell(opponentid, 'Found opponent...')
                                gamethread.delayed(1, player_zone, (int(userid), int(player.attributes['teamid']), int(opponentid), int(opponentplayer.attributes['teamid'])))
                            else:
                                es.centertell(userid, 'Searching opponent...')
                                gamethread.delayed(1, player_queue, (int(userid), int(knife_arena_players[userid]['spawns'])))
                            
def est_spawn(userid):
    if es.exists('userid', userid):
        player = playerlib.getPlayer(userid)
        if (int(player.attributes['isdead']) == 1) and (int(player.attributes['teamid']) > 1):
            es.server.cmd('est_spawn '+str(userid))

def est_getentname(index):
    result = es.ServerVar('_tempcore', '')
    es.server.cmd('est_getentname _tempcore '+str(index))
    return str(result)

def player_spawn(event_var):
    global knife_arena_reset, knife_arena_areas, knife_arena_waitqueue
    userid = int(event_var['userid'])
    if not userid in knife_arena_players:
        knife_arena_players[userid] = {'lives':int(knife_arena_limit),'fight':False,'spawns':0,'time':time.time()}
    if knife_arena_reset:
        knife_arena_reset = False
        knife_arena_areas = {}
        areacount = 0
        targets = es.createentitylist('info_target')
        for point in targets:
            areaid = 0
            areateam = 0
            targetname = est_getentname(point)
            if targetname[-5:] == '_ka_t':
                areaid = int(targetname[:-5])
                areateam = 2
            elif targetname[-6:] == '_ka_ct':
                areaid = int(targetname[:-6])
                areateam = 3
            if (areaid > 0) and (areateam > 1):
                areacount = areacount + 1
                if not areaid in knife_arena_areas:
                    knife_arena_areas[areaid] = {}
                if not areateam in knife_arena_areas[areaid]:
                    knife_arena_areas[areaid][areateam] = {}
                knife_arena_areas[areaid]['inuse'] = False
                knife_arena_areas[areaid]['time'] = int(knife_arena_timelimit)
                knife_arena_areas[areaid][areateam]['origin'] = str(es.getindexprop(point, 'CBaseEntity.m_vecOrigin')).split(',')
                knife_arena_areas[areaid][areateam]['rotation'] = str(es.getindexprop(point, 'CBaseEntity.m_angRotation')).split(',')
        if int(areacount) != (len(knife_arena_areas)*2):
            es.msg('#multi', '#green[KnifeArena] #lightgreenThe areas on this map are screwed up!')
        else:
            es.msg('#multi', '#green[KnifeArena] #lightgreenFound '+str(int(areacount/2))+' areas!')
        if not knife_arena_areas:
            es.msg('#multi', '#green[KnifeArena] #lightgreenNo areas are found on this map!')
    if knife_arena_areas:
        es.ServerCommand('est_deleteragdolls')
        es.ServerCommand('est_removeidle weapon')
        player = playerlib.getPlayer(userid)
        if (int(player.attributes['teamid']) == 2) or int(int(player.attributes['teamid']) == 3):
            player.set('cash', '0')
            player.set('health', int(knife_arena_health))
            knife_arena_players[userid]['spawns'] = knife_arena_players[userid]['spawns'] + 1
            knife_arena_players[userid]['fight'] = False
            knife_arena_players[userid]['start'] = es.getplayerlocation(userid)
            knife_arena_players[userid]['time'] = time.time()
            knife_arena_players[userid]['opponent'] = 0
            es.ServerCommand('est_stripplayer '+str(userid)+' 1')
            es.centertell(userid, 'Searching opponent...')
            gamethread.delayed(1, player_wait, (int(userid), int(player.attributes['teamid'])))
            gamethread.delayed(1, player_queue, (int(userid), int(knife_arena_players[userid]['spawns'])))

def player_death(event_var):
    userid = int(event_var['userid'])
    attackerid = int(event_var['attacker'])
    if knife_arena_areas:
        if (userid > 0) and (attackerid > 0) and (userid in knife_arena_players) and (attackerid in knife_arena_players):
            areaid = knife_arena_players[userid]['fight']
            if areaid > 0:
                knife_arena_areas[areaid]['time'] = int(knife_arena_timelimit)
                knife_arena_players[userid]['fight'] = 'dead'
                knife_arena_players[userid]['time'] = time.time()
                gamethread.delayed(5, player_unlock, (areaid))
                if int(knife_arena_limit) > 0:
                    knife_arena_players[userid]['lives'] = knife_arena_players[userid]['lives'] - 1
                if int(event_var['es_userteam']) != int(event_var['es_attackerteam']):
                    knife_arena_scores[int(event_var['es_attackerteam'])] = knife_arena_scores[int(event_var['es_attackerteam'])] + 1
                    es.ServerCommand('score set #t '+str(knife_arena_scores[2]))
                    es.ServerCommand('score set #ct '+str(knife_arena_scores[3]))
                    player = playerlib.getPlayer(userid)
                    player.set('cash', '0')
                    opponentplayer = playerlib.getPlayer(attackerid)
                    opponentplayer.set('cash', '0')
                    opponentplayer.set('health', int(knife_arena_health))
                    es.centertell(userid, 'Lost! Respawning soon...')
                    es.centertell(attackerid, 'Win! Searching new opponent...')
                    gamethread.delayed(2, player_reset, (attackerid, int(knife_arena_players[attackerid]['spawns'])))
                    if ((int(knife_arena_players[userid]['lives']) > 0) or (int(knife_arena_limit) == 0)):
                        gamethread.delayed(7, est_spawn, (userid))
                else:
                    attackerid = int(knife_arena_players[userid]['opponent'])
                    player = playerlib.getPlayer(userid)
                    player.set('cash', '0')
                    opponentplayer = playerlib.getPlayer(attackerid)
                    opponentplayer.set('cash', '0')
                    opponentplayer.set('health', int(knife_arena_health))
                    es.centertell(userid, 'Stupid! Respawning soon...')
                    es.centertell(attackerid, 'Win! Searching new opponent...')
                    gamethread.delayed(2, player_reset, (attackerid, int(knife_arena_players[attackerid]['spawns'])))
                    if ((int(knife_arena_players[userid]['lives']) > 0) or (int(knife_arena_limit) == 0)):
                        gamethread.delayed(7, est_spawn, (userid))

def player_disconnect(event_var):
    userid = int(event_var['userid'])
    if knife_arena_areas:
        if userid in knife_arena_players:
            attackerid = int(knife_arena_players[userid]['opponent'])
            if attackerid > 0:
                opponentplayer = playerlib.getPlayer(attackerid)
                opponentplayer.set('cash', '0')
                opponentplayer.set('health', int(knife_arena_health))
                es.centertell(attackerid, 'Searching new opponent...')
                gamethread.delayed(3, player_reset, (attackerid, int(knife_arena_players[attackerid]['spawns'])))
                del knife_arena_players[userid]

def player_team(event_var):
    userid = int(event_var['userid'])
    if knife_arena_areas:
        if userid in knife_arena_waitqueue[2]:
            knife_arena_waitqueue[2].remove(userid)
        if userid in knife_arena_waitqueue[3]:
            knife_arena_waitqueue[3].remove(userid)
        if userid in knife_arena_players:
            if int(event_var['disconnect']) == 0:
                player = playerlib.getPlayer(userid)
                player.set('cash', '0')
                if int(event_var['team']) > 1:
                    if ((int(knife_arena_players[userid]['lives']) > 0) or (int(knife_arena_limit) == 0)):
                        gamethread.delayed(3, est_spawn, (userid))
                else:
                    knife_arena_players[userid]['fight'] = False
                    knife_arena_players[userid]['time'] = time.time()
                    knife_arena_players[userid]['opponent'] = 0

def watch():
    es.ServerCommand('es_xfire '+str(es.getcmduserid())+' !self setteam 1')
    
def fight():
    userid = int(es.getcmduserid())
    player = playerlib.getPlayer(userid)
    if int(player.attributes['teamid']) < 2:
        if len(playerlib.getUseridList('#t')) < len(playerlib.getUseridList('#ct')):
            es.ServerCommand('es_xfire '+str(es.getcmduserid())+' !self setteam 2')
        else:
            es.ServerCommand('es_xfire '+str(es.getcmduserid())+' !self setteam 3')
    if int(player.attributes['isdead']) == 1:
        if ((int(knife_arena_players[userid]['lives']) > 0) or (int(knife_arena_limit) == 0)):
            es.ServerCommand('est_spawn '+str(userid))
